/*
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation.  All Rights
 * Reserved.
 *
 * Contributor(s):
 *   EVM-Group <evm@s3.uni-essen.de>
 */


// **************************************************************************
// ************************* EVM-DB Functions *******************************
// **************************************************************************
// +++ FOR PUTTING MOZILLA DATA INTO DATABASE +++
// * for English description please read functions' prologues
//
// Tim Franz:
// evmInitConnection						-->	realisiert DB-Anbindung
//
// Kai Henning:
// evmGetMailAdressFromRecipient			--> extrahiert Mailadresse aus Recipient
// evmGetRealNameFromRecipient				--> extrahiert Realname aus Recipient
// evmIsEntryAvailableInDB					--> prft, ob der Rckgabewert der SQL-
//												Anfrage zu einem Ergebnis gefhrt hat
// evmReturnAvailableDBEntry				--> gibt den Eintrag (sofern vorhanden) in
//												Form eines Integer zurck
// evmSplitter(b,weg,hin)						--> ersetzt Teile des bergebenen Strings b
// evmParseBody(rohling)					--> ersetzt 'kritische' Strings fr das
//												Schreiben in die DB
// evmParseEmpty(rohling)					--> entfernt 'kritische' Zeichen aus String
// evmDoQuery								--> schreibt Mail in DB
//
// **************************************************************************
// Version 07.02.2004 (Kai Henning, Andr Grothues, Tim Franz)
// **************************************************************************


  const alias = "urn:aliases:mozdbtest";
  var connection;
  var result;

  // *************************************************************************
  // function:		evmInitConnection()
  // status:		finished
  // author:		Tim Franz
  // *************************************************************************
  // establishes a connection to the mozdbtest-DB-Alias
  function evmInitConnection()
  {
	try
	{
      var service = Components.classes["@mozilla.org/sql/service;1"].getService(Components.interfaces.mozISqlService);
      if (!service.hasAlias(alias))
      {
        alert("The alias for the sqltest is not defined.");
        return;
      }
      try
      {
        connection = service.getConnection(alias);
      }
      catch (ex)
      {
        alert(service.errorMessage);
        alert("connection failed");
        return;
      }
    }
    catch(ex)
    {
      alert("EVM Error in function evmInitConnection: " + ex);
    }
  }


  // *************************************************************************
  // function:		evmIsInteger(pruef)
  // compare:		function evmIsInteger(pruef) in evmxulK.js!!!
  // *************************************************************************
  function evmIsInteger(pruef)
  {
    return (((pruef+5)+5) == (pruef+10));
  }


  // *************************************************************************
  // function:		evmGetMailAdressFromRecipient(ma)
  // status:		finished
  // author:		Kai Henning
  // *************************************************************************
  // splits up a string at the blank (" ") and searches for the part containing
  // the @. After that, the functions checks whether there are < or > in that
  // part. If so, it is taken out too, and the mailaddress (e.g. xyz@provider.com)
  // is then returned only
  function evmGetMailAdressFromRecipient(ma)
  {
	try
	{
      auto = ma.split(" ");
      for(var i = 0; i < auto.length; i++)
      {
        if(auto[i].indexOf("@") >= 0)
        {
	      var t = auto[i];
        }
      }
      if((t.indexOf("<") >= 0) && (t.indexOf(">") >= 0))
      {
        t = t.substring(t.indexOf("<")+1,t.indexOf(">"));
      }
      return t;
    }
    catch(ex)
    {
      //alert("EVM Error in function evmGetMailAdressFromRecipient: " + ex);
      return "evmError";
    }
  }


  // *************************************************************************
  // function:		evmGetRealNameFromRecipient(mn)
  // status:		finished
  // author:		Kai Henning
  // *************************************************************************
  // Splits up a string at the blank (" ") and searches for the parts NOT
  // containing the @. After that, the functions checks whether there are
  // " or ' in those parts. If so, these things taken out too, and the
  // single 'clean' parts are put together again and returned.
  // if the first splits fails, it is likely that no realname is in the
  // string at all, so we want to store the mailaddress in the mailaddress
  // and in the realname column of the DB (our functions provide updating
  // in case we sometimes will get a realname for a saved mailaddress!),
  // so we call the function evmGetMailAdressFromRecipient with the original
  // parameter.
  // If the first split just has one result, we had a mailaddress in the
  // string only, so we either return it (if it does not contain < or >)
  // or call evmGetMailAdressFromRecipient to 'clean' the mailaddress.
  // In case we screwed something up, we check in the end whether we got
  // an empty string... in that case, we rather want to return the
  // mailaddress (instead of the realname which we obviously couldn't
  // handle), so we call the evmGetMailAdressFromRecipient with the original
  // parameter.
  function evmGetRealNameFromRecipient(mn)
  {
	try
	{
      auto = mn.split(" ");
      if(auto.length == 0) return evmGetMailAdressFromRecipient(mn);
      if(auto.length == 1)
      {
        if(auto[0].indexOf("<") < 0) return auto[0];
        else return evmGetMailAdressFromRecipient(mn);
      }
	  // *** filling all parts not containing an @ into a new array
      var bestandteile = [];
      var b = -1;
      for(var i = 0; i < auto.length; i++)
      {
        if(auto[i] != "" && auto[i] != " " && (auto[i].indexOf("@") == -1))
        {
          b++;
          bestandteile[b] = auto[i];
        }
      }
	  // *** checking and deleting " and '
      for(var i = 0; i <= b; i++)
      {
        if(bestandteile[i].indexOf("\"") >= 0)
        {
          var x = bestandteile[i].split("\"");
          var y = "";
	      for(var j = 0; j < x.length; j++)
	      {
	        if(x[j] != " " && x[j] != "") y = y + x[j];
	      }
	      bestandteile[i] = y;
        }
        if(bestandteile[i].indexOf("\'") >= 0)
        {
          var x = bestandteile[i].split("\'");
          var y = "";
	      for(var j = 0; j < x.length; j++)
	      {
	        if(x[j] != " " && x[j] != "") y = y + x[j];
	      }
	      bestandteile[i] = y;
        }
      }
	  // *** putting parts together again
      var t = "";
      for(var i = 0; i <= b; i++)
      {
        t = t + bestandteile[i];
        if(i < b) t = t + " ";
      }
	  // *** if we got an empty string in the end, let's better return the mailaddress...
      if(t == "" || t == " ") return evmGetMailAdressFromRecipient(mn);
      return t;
    }
    catch(ex)
    {
      alert("EVM Error in function evmGetRealNameFromRecipient: " + ex);
      return "evmError";
    }
  }


  // *************************************************************************
  // function:		evmIsEntryAvailableInDB(r)
  // status:		finished
  // author:		Kai Henning
  // *************************************************************************
  // when we made a DB query, we want to know whether there are any results
  // which we got back from it
  // returns a boolean value
  function evmIsEntryAvailableInDB(r)
  {
	try
	{
      var enumerator = r.enumerate();
      try
      {
        enumerator.first();
      }
      catch(ex)
      {
        return false;
      }
      var ergebnis = enumerator.getVariant(0);
      var schonda = parseInt(ergebnis);
      return evmIsInteger(schonda);
    }
    catch(ex)
    {
      // alert("EVM Error in function evmIsEntryAvailableInDB: " + ex);
      return false;
    }
  }


  // *************************************************************************
  // function:		evmReturnAvailableDBEntry(r)
  // status:		finished
  // author:		Kai Henning
  // *************************************************************************
  // mostly used in combination with the function evmIsEntryAvailableInDB(r)
  // to send back the first column entry of the first row (we use it mostly
  // to get the highest ID which already are in the DB in a certain table
  function evmReturnAvailableDBEntry(r)
  {
	try
	{
      var enumerator = r.enumerate();
      enumerator.first();
      var ergebnis = enumerator.getVariant(0);
      var schonda = parseInt(ergebnis);
      return schonda;
    }
    catch(ex)
    {
      alert("EVM Error in function evmReturnAvailableDBEntry: " + ex);
    }
  }


  // *************************************************************************
  // function:		evmSplitter(b,weg,hin)
  // status:		finished
  // author:		Kai Henning
  // *************************************************************************
  // splits up a string 'b' at the string specified by 'weg' and puts all
  // pieces together again using the 'hin' string between it
  function evmSplitter(b,weg,hin)
  {
	try
	{
      var erg = b.split(weg);
      var t = "";
      for(var i = 0; i < erg.length; i++)
      {
        t = t + erg[i];
        if(i < erg.length-1) t = t + hin;
      }
      return t;
    }
    catch(ex)
    {
      alert("EVM Error in function evmSplitter: " + ex);
    }
  }


  // *************************************************************************
  // function:		evmParseBody(rohling)
  // status:		finished
  // author:		Kai Henning
  // *************************************************************************
  // to write the body to the DB, it is necessary to replace characters like
  // ', " and < by strings that won't disturb the SQL statement. For that, we
  // chose a number plus 'evm' plus a second number (we picked the numbers at
  // random hoping this string won't exist in the real mailbody!)
  function evmParseBody(rohling)
  {
	try
	{
      var newbody = evmSplitter(rohling,"\"","35evmDH47");
      newbody = evmSplitter(newbody,"'","75evmSH89");
      newbody = evmSplitter(newbody,"<","21evmLT80");
      var anf = "&quot;";
      newbody = evmSplitter(newbody,anf,"01evmAF99");
      return newbody;
    }
    catch(ex)
    {
      alert("EVM Error in function evmParseBody: " + ex);
    }
  }


  // *************************************************************************
  // function:		evmParseEmpty(rohling)
  // status:		finished
  // author:		Kai Henning
  // *************************************************************************
  // deletes all characters non-conform for the DB from a string
  function evmParseEmpty(rohling)
  {
	try
	{
      var newbody = evmSplitter(rohling,"\"","");
      newbody = evmSplitter(newbody,"'","");
      newbody = evmSplitter(newbody,"<","");
      var anf = "&quot;";
      newbody = evmSplitter(newbody,anf,"");
      return newbody;
    }
    catch(ex)
    {
      alert("EVM Error in function evmParseEmpty: " + ex);
    }
  }

  // *************************************************************************
  // function:		evmDoQuery(m)
  // status:		finished
  // author:		Kai Henning / Andr Grothues
  // *************************************************************************
  // after having checked whether mail is in DB already, we insert the mail
  // step by step
  // please notice that we only insert the msgid into the header_category table
  // properly because later functions will need this uncomplete function to know
  // which of the tabboxes to open etc
  function evmDoQuery(m)
  {
    try
    {
      evmInitConnection();
	  m['subject'] = evmParseEmpty(m['subject'])
      ////////// Checking whether mail is in DB already: //////////
      try
      {
	    query = "select * from header where (msgid ='" + m['messageid'] + "')";
        result = connection.executeQuery(query);
      }
      catch(ex) { }
      try
      {
        if(evmIsEntryAvailableInDB(result))
        {
          //alert("Mail '" + m['subject'] + "' by " + m['from'] + " to " + m['to'] + " is already in the EVM!!!");
		  waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; Mail is already in the EVM");
          return;
        }
      }
      catch (ex)
      {
        if(connection.errormessage)
        {
          //alert("Database Query failed: " + connection.errormessage);
		  waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
		}
      }

      ////////// inserting recipient(s) - To: //////////
      try
      {
        var query = "select max(id) from mailaddress";
        sql_headerfields = "address, id, realname";
        var result = connection.executeQuery(query);
      }
      catch(ex) { }
      try
      {
        if(!evmIsEntryAvailableInDB(result)) recipid = 0;
        else recipid = evmReturnAvailableDBEntry(result);
        recipid = recipid + 1;

	    var recip_array = m['to'].split(",");
	    for(var i = 0; i < recip_array.length; i++)
	    {
	      mailaddr = evmGetMailAdressFromRecipient(recip_array[i]).toLowerCase();
	      mailname = evmGetRealNameFromRecipient(recip_array[i]);
	      try
	      {
	        query = "select id, realname from mailaddress where (address ='" + mailaddr + "')";
            result = connection.executeQuery(query);
          }
          catch(ex) { }
          try
          {
            enumerator = result.enumerate();
		    if(result.rowCount > 0)
		    {
              enumerator.first();
              ergebnis = enumerator.getVariant(0);
		      if(result.rowCount > 0) oldrealname = enumerator.getVariant(1);
              vorhandeneid = parseInt(ergebnis);
		    }
		    else vorhandeneid = "xxx";
          }
          catch (ex)
          {
            if(connection.errormessage)
            {
              //alert("Inserting " + mailname + "<" + mailaddr + "> Data into Mailaddress Table caused this error: " + connection.errormessage);
			  waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
			}
          }
          if(!evmIsInteger(vorhandeneid))
	      {
            sql_headerfields = "address, id, realname";
            sql_headervalues = "'" + mailaddr + "'," + recipid + ",'" + mailname + "'";
	        try
  	        {
              query = "INSERT INTO mailaddress (" + sql_headerfields + ") VALUES (" + sql_headervalues + ")";
		      vorhandeneid = recipid;
              recipid = recipid + 1;
              result = connection.executeQuery(query);
            }
            catch (ex)
            {
              if(connection.errormessage)
              {
              	//alert("Inserting " + mailname + "<" + mailaddr + "> Data into Mailaddress Table caused this error: " + connection.errormessage);
				waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
			  }
            }
          } // if
	      else
	      {
	        if((oldrealname == mailaddr) && (mailaddr != mailname))
		    {
	          try
  	          {
                query = "UPDATE mailaddress SET realname = '" + mailname + "' WHERE (id =" + vorhandeneid + ")";
                result = connection.executeQuery(query);
              }
              catch (ex)
              {
              }
		    }
	      }
	      try
	      {
	        sql_headerfields = "msgID, mailaddressID, mailadrType";
	        sql_headervalues= "'" + m['messageid'] + "'," + vorhandeneid + ",'0to'";
	        query = "INSERT INTO header_mailaddress (" + sql_headerfields + ") VALUES (" + sql_headervalues + ")";
	        connection.executeQuery(query);
  	      }
	      catch (ex)
	      {
	        if(connection.errormessage)
	        {
	          //alert("Inserting into Header_Mailaddress Table caused this error: " + connection.errormessage);
			  waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
			}
	      }
	    } // for
      }
      catch (ex)
      {
        if(connection.errormessage)
        {
          //alert("Inserting Recipients Data into Mailaddress Table caused this error: " + connection.errormessage);
		  waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
		}
      }

      ////////// inserting recipient(s) - CC: //////////
      if(m['cc'])
      {
        try
        {
  	      evmCCList = m['cc'];
          var query = "select max(id) from mailaddress";
          sql_headerfields = "address, id, realname";
          var result = connection.executeQuery(query);
        }
        catch(ex) { }
        try
        {
          if(!evmIsEntryAvailableInDB(result)) recipid = 0;
          else recipid = evmReturnAvailableDBEntry(result);
          recipid = recipid + 1;

	      var recip_array = evmCCList.split(",");
	      for(var i = 0; i < recip_array.length; i++)
	      {
	        mailaddr = evmGetMailAdressFromRecipient(recip_array[i]).toLowerCase();
	        mailname = evmGetRealNameFromRecipient(recip_array[i]);
	        try
	        {
	          query = "select id, realname from mailaddress where (address ='" + mailaddr + "')";
              result = connection.executeQuery(query);
            }
            catch(ex) { }
            try
            {
              enumerator = result.enumerate();
		      if(result.rowCount > 0)
		      {
                enumerator.first();
                ergebnis = enumerator.getVariant(0);
		        if(result.rowCount > 0) oldrealname = enumerator.getVariant(1);
                vorhandeneid = parseInt(ergebnis);
		      }
		      else vorhandeneid = "xxx";
            }
            catch (ex)
            {
              if(connection.errormessage)
              {
              	//alert("Inserting " + mailname + "<" + mailaddr + "> Data into Mailaddress Table caused this error: " + connection.errormessage);
				waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
			  }
            }
            if(!evmIsInteger(vorhandeneid))
	        {
              sql_headerfields = "address, id, realname";
              sql_headervalues = "'" + mailaddr + "'," + recipid + ",'" + mailname + "'";
	          try
  	          {
                query = "INSERT INTO mailaddress (" + sql_headerfields + ") VALUES (" + sql_headervalues + ")";
		        vorhandeneid = recipid;
                recipid = recipid + 1;
                result = connection.executeQuery(query);
              }
              catch (ex)
              {
                if(connection.errormessage)
                {
                  //alert("Inserting " + mailname + "<" + mailaddr + "> Data into Mailaddress Table caused this error: " + connection.errormessage);
				  waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
				}
              }
            } // if
	        else
	        {
	          if((oldrealname == mailaddr) && (mailaddr != mailname))
		      {
	            try
  	            {
                  query = "UPDATE mailaddress SET realname = '" + mailname + "' WHERE (id =" + vorhandeneid + ")";
                  result = connection.executeQuery(query);
                }
                catch (ex)
                {
                }
		      }
	        }
	        try
	        {
	          sql_headerfields = "msgID, mailaddressID, mailadrType";
	          sql_headervalues= "'" + m['messageid'] + "'," + vorhandeneid + ",'1cc'";
	          query = "INSERT INTO header_mailaddress (" + sql_headerfields + ") VALUES (" + sql_headervalues + ")";
	          connection.executeQuery(query);
  	        }
	        catch (ex)
	        {
	          if(connection.errormessage)
	          {
	          	//alert("Inserting into Header_Mailaddress Table caused this error: " + connection.errormessage);
				waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
			  }
	        }
	      } // for
        }
        catch (ex)
        {
          if(connection.errormessage)
          {
          	//alert("Inserting Recipients Data into Mailaddress Table caused this error: " + connection.errormessage);
			waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
		  }
        }
      }

      ////////// inserting recipient(s) - BCC: //////////
      if(m['bcc'])
      {
        try
        {
          evmBCCList = m['bcc'];
          var query = "select max(id) from mailaddress";
          sql_headerfields = "address, id, realname";
          var result = connection.executeQuery(query);
        }
        catch(ex) { }
        try
        {
          if(!evmIsEntryAvailableInDB(result)) recipid = 0;
          else recipid = evmReturnAvailableDBEntry(result);
          recipid = recipid + 1;

	      var recip_array = evmBCCList.split(",");
	      for(var i = 0; i < recip_array.length; i++)
	      {
	        mailaddr = evmGetMailAdressFromRecipient(recip_array[i]).toLowerCase();
	        mailname = evmGetRealNameFromRecipient(recip_array[i]);
	        try
	        {
	          query = "select id, realname from mailaddress where (address ='" + mailaddr + "')";
              result = connection.executeQuery(query);
            }
            catch(ex) { }
            try
            {
              enumerator = result.enumerate();
		      if(result.rowCount > 0)
		      {
                enumerator.first();
                ergebnis = enumerator.getVariant(0);
		        if(result.rowCount > 0) oldrealname = enumerator.getVariant(1);
                vorhandeneid = parseInt(ergebnis);
		      }
		      else vorhandeneid = "xxx";
            }
            catch (ex)
            {
              if(connection.errormessage)
              {
              	//alert("Inserting " + mailname + "<" + mailaddr + "> Data into Mailaddress Table caused this error: " + connection.errormessage);
				waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
			  }
            }
            if(!evmIsInteger(vorhandeneid))
	        {
              sql_headerfields = "address, id, realname";
              sql_headervalues = "'" + mailaddr + "'," + recipid + ",'" + mailname + "'";
	          try
  	          {
                query = "INSERT INTO mailaddress (" + sql_headerfields + ") VALUES (" + sql_headervalues + ")";
		        vorhandeneid = recipid;
                recipid = recipid + 1;
                result = connection.executeQuery(query);
              }
              catch (ex)
              {
                if(connection.errormessage)
                {
                  //alert("Inserting " + mailname + "<" + mailaddr + "> Data into Mailaddress Table caused this error: " + connection.errormessage);
				  waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
				}
              }
            } // if
	        else
	        {
	          if((oldrealname == mailaddr) && (mailaddr != mailname))
		      {
	            try
  	            {
                  query = "UPDATE mailaddress SET realname = '" + mailname + "' WHERE (id =" + vorhandeneid + ")";
                  result = connection.executeQuery(query);
                }
                catch (ex)
                {
                }
		      }
	        }
	        try
	        {
	          sql_headerfields = "msgID, mailaddressID, mailadrType";
	          sql_headervalues= "'" + m['messageid'] + "'," + vorhandeneid + ",'2bcc'";
	          query = "INSERT INTO header_mailaddress (" + sql_headerfields + ") VALUES (" + sql_headervalues + ")";
	          connection.executeQuery(query);
  	        }
	        catch (ex)
	        {
	          if(connection.errormessage)
	          {
	          	//alert("Inserting into Header_Mailaddress Table caused this error: " + connection.errormessage);
				waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
			  }
	        }
	      } // for
        }
        catch (ex)
        {
          if(connection.errormessage)
          {
          	//alert("Inserting Recipients Data into Mailaddress Table caused this error: " + connection.errormessage);
			waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
		  }
        }
      }

      ////////// inserting sender: //////////
      try
      {
        var query = "select max(id) from mailaddress";
        sql_headerfields = "address, id, realname";
        var result = connection.executeQuery(query);
      }
      catch(ex) { }
      try
      {
	    if(!evmIsEntryAvailableInDB(result)) recipid = 0;
	    else recipid = evmReturnAvailableDBEntry(result);
        recipid = recipid + 1;

  	    mailaddr = evmGetMailAdressFromRecipient(m['from']).toLowerCase();
	    mailname = evmGetRealNameFromRecipient(m['from']);
	    try
	    {
	      query = "select max(id) from mailaddress where (address ='" + mailaddr + "')";
          result = connection.executeQuery(query);
        }
        catch(ex) { }
        try
        {
          enumerator = result.enumerate();
          enumerator.first();
          ergebnis = enumerator.getVariant(0);
          vorhandeneid = parseInt(ergebnis);
        }
        catch (ex)
        {
          if(connection.errormessage)
          {
          	//alert("Inserting " + mailname + "<" + mailaddr + "> Data into Mailaddress Table caused this error: " + connection.errormessage);
			waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
		  }
        }
        if(!evmIsInteger(vorhandeneid))
	    {
          sql_headerfields = "address, id, realname";
          sql_headervalues= "'" + mailaddr + "'," + recipid + ",'" + mailname + "'";
	      try
  	      {
            query = "INSERT INTO mailaddress (" + sql_headerfields + ") VALUES (" + sql_headervalues + ")";
	   	    vorhandeneid = recipid;
            recipid = recipid + 1;
            result = connection.executeQuery(query);
          }
          catch (ex)
          {
            if(connection.errormessage)
            {
              //alert("Inserting " + mailname + "<" + mailaddr + "> Data into Mailaddress Table caused this error: " + connection.errormessage);
			  waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
			}
          }
	    } // if
	    else
	    {
	      if((oldrealname == mailaddr) && (mailaddr != mailname))
		  {
	        try
  	        {
              query = "UPDATE mailaddress SET realname = '" + mailname + "' WHERE (id =" + vorhandeneid + ")";
              result = connection.executeQuery(query);
            }
            catch (ex)
            {
            }
		  }
	    }
      }
      catch (ex)
      {
        if(connection.errormessage)
        {
          //alert("Inserting Sender Data into Mailaddress Table caused this error: " + connection.errormessage);
		  waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
		}
      }

      ////////// inserting reply-to-address(es): //////////
	  var reply2 = "";
	  for(element in m)
	  {
	    if(element.toLowerCase().indexOf("reply") >= 0 && element.toLowerCase().indexOf("to") >= 0 && element.toLowerCase().indexOf("in") < 0) reply2 = element;
	  }
	  if(reply2 != "")
	  {
        try
      	{
	  	  evmReplyList = m[reply2];
    	  var query = "select max(id) from mailaddress";
      	  sql_headerfields = "address, id, realname";
      	  var result = connection.executeQuery(query);
    	}
    	catch(ex) { }
    	try
    	{
	      if(!evmIsEntryAvailableInDB(result)) replyid = 0;
	      else replyid = evmReturnAvailableDBEntry(result);
	      replyid = replyid + 1;
		  var recip_array = evmReplyList.split(",");
		  for(var i = 0; i < recip_array.length; i++)
		  {
		    mailaddr = evmGetMailAdressFromRecipient(recip_array[i]).toLowerCase();
		    mailname = evmGetRealNameFromRecipient(recip_array[i]);
		    try
		    {
		      query = "select id, realname from mailaddress where (address ='" + mailaddr + "')";
	          result = connection.executeQuery(query);
	        }
    	    catch(ex) { }
        	try
	        {
	          enumerator = result.enumerate();
			  if(result.rowCount > 0)
			  {
	            enumerator.first();
	            ergebnis = enumerator.getVariant(0);
			    if(result.rowCount > 0) oldrealname = enumerator.getVariant(1);
	            vorhandenerepid = parseInt(ergebnis);
			  }
			  else vorhandenerepid = "xxx";
        	}
        	catch (ex)
        	{
          	  if(connection.errormessage)
          	  {
          	  	//alert("Inserting " + mailname + "<" + mailaddr + "> Data into Mailaddress Table caused this error: " + connection.errormessage);
				waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
			  }
        	}
        	if(!evmIsInteger(vorhandenerepid))
		    {
	          sql_headerfields = "address, id, realname";
	          sql_headervalues = "'" + mailaddr + "'," + replyid + ",'" + mailname + "'";
		      try
	  	      {
	            query = "INSERT INTO mailaddress (" + sql_headerfields + ") VALUES (" + sql_headervalues + ")";
			    vorhandenerepid = replyid;
	            replyid = recipid + 1;
	            result = connection.executeQuery(query);
	          }
	          catch (ex)
	          {
	            if(connection.errormessage)
	            {
	              //alert("Inserting " + mailname + "<" + mailaddr + "> Data into Mailaddress Table caused this error: " + connection.errormessage);
				  waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
				}
	          }
	        } // if
		    try
		    {
		      sql_headerfields = "msgID, mailaddressID, mailadrType";
		      sql_headervalues= "'" + m['messageid'] + "'," + vorhandenerepid + ",'3replyto'";
		      query = "INSERT INTO header_mailaddress (" + sql_headerfields + ") VALUES (" + sql_headervalues + ")";
		      connection.executeQuery(query);
	  	    }
		    catch (ex)
		    {
			  if(connection.errormessage)
			  {
			  	//alert("Inserting into Header_Mailaddress Table caused this error: " + connection.errormessage);
				waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
			  }
		    }
	  	  } // for
      	}
      	catch (ex)
    	{
      	  if(connection.errormessage)
      	  {
      	  	//alert("Inserting Recipients Data into Mailaddress Table caused this error: " + connection.errormessage);
			waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
		  }
    	}
      }

      ////////// inserting of body: //////////
      try
      {
        var query = "select max(id) from body";
        var result = connection.executeQuery(query);
      }
      catch(ex)
      {
      }
      try
      {
	    if(!evmIsEntryAvailableInDB(result)) bodyid = 0;
        else bodyid = evmReturnAvailableDBEntry(result);
        bodyid = bodyid + 1;
        var wonderbody = evmParseBody(m['body']);
        sql_headerfields = "id, bodytext, msgid";
        sql_headervalues= bodyid + ",'" + wonderbody + "','" + m['messageid'] + "'";
        query = "INSERT INTO body (" + sql_headerfields + ") VALUES (" + sql_headervalues + ")";
	    var result = connection.executeQuery(query);
      }
      catch (ex)
      {
        if(connection.errormessage)
        {
          //alert("Inserting Body into Body Table caused this error: " + connection.errormessage);
		  waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
		}
      }

	  ////////// preparing category choice: //////////
	  try
	  {
        tmpcid = -1232006;
        sql_headerfields = "msgid, categoryid";
        sql_headervalues= "'" + m['messageid'] + "'," + tmpcid;
        query = "INSERT INTO header_category (" + sql_headerfields + ") VALUES (" + sql_headervalues + ")";
	    var result = connection.executeQuery(query);
      }
      catch (ex)
      {
      }



      ////////// inserting headers: //////////
      try
      {
	    var refer = "";
	    for(element in m)
	    {
	      if(element.toLowerCase().indexOf("refere") >= 0 || (element.toLowerCase().indexOf("reply") >= 0 && element.toLowerCase().indexOf("to") >= 0 && element.toLowerCase().indexOf("in") >= 0)) refer = m[element];
	    }
	    var prio = 3;
	    for(element in m)
	    {
	      if(element.toLowerCase().indexOf("priority") >= 0)
	      {
	        prio = parseInt(m[element]);
	        if(!evmIsInteger(prio)) prio = 3;
	      }
	    }
        var query = "select max(id) from header";
        var result = connection.executeQuery(query);
      }
      catch(ex) { }
      try
      {
	    if(!evmIsEntryAvailableInDB(result)) headerid = 0;
	    else headerid = evmReturnAvailableDBEntry(result);
	    if(m['subject'] == "" || m['subject'] == " ") m['subject'] = " ";
        headerid = headerid + 1;
        sql_headerfields = "id, msgid, sender, subject, date, time, priority, ref, tstamp";
        sql_headervalues= headerid + ",'" + m['messageid'] + "'," + vorhandeneid + ",'" + m['subject'] + "','" + m['Kdate'] + "','" + m['Ktime'] + "'," + prio + ",'" + refer + "','" + m["trex"] + "'";
        query = "INSERT INTO header (" + sql_headerfields + ") VALUES (" + sql_headervalues + ")";
        var result = connection.executeQuery(query);
      }
      catch (ex)
      {
        if(connection.errormessage)
        {
          //alert("Inserting Sender into Mailaddress Table caused this error: " + connection.errormessage);
		  waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; " +  connection.errormessage);
		}
      }



      ////////// inserting attachment(s) [WITHOUT path or data!!!]: //////////
      var query = "select max(id) from attachment";
      try
      {
        var result = connection.executeQuery(query);
      }
      catch(ex)
      {
      }
      if(!evmIsEntryAvailableInDB(result)) attid = 0;
      else attid = evmReturnAvailableDBEntry(result);
      attid++;
      var sql_headerfields = "id, name, type, location, msgid";
	  for(var i = 1; i <= m["numAttachments"]; i++)
	  {
	    try
	    {
          sql_headervalues= "'" + attid + "','" + m["evmA" + i + "displayName"] + "','" + m["evmA" + i + "contentType"] + "','XXX','" + m["messageid"] + "'";
          attid++;
          query = "INSERT INTO attachment (" + sql_headerfields + ") VALUES (" + sql_headervalues + ")";

	      connection.executeQuery(query);
        }
        catch (ex)
        {
        }
      }



      ////////// inserting rest of headers: //////////
      m["Ktime"]		    = "evmEmpty";
      m["Kdate"]		    = "evmEmpty";
      m["trex"]		    = "evmEmpty";
      m["mURI"]           = "evmEmpty";
      m["fURI"]           = "evmEmpty";
      m["sURI"]           = "evmEmpty";
      m["msgsize"]        = "evmEmpty";
      m["offlmsgsize"]    = "evmEmpty";
      m["Charset"]        = "evmEmpty";
	  m["body"]			= "evmEmpty";
      for(e in m)
      {
	    if(e.indexOf("evmA") == 0) m[e] = "evmEmpty";
	  }
	  var restheader = "";
	  for(e in m)
	  {
	    if(restheader != "" && m[e] != "evmEmpty") restheader = restheader + ",,,";
	    if(m[e] != "evmEmpty") restheader = restheader + e + ":::" + m[e];
	  }
	  try
	  {
        query = "UPDATE header SET meta= '" + restheader + "' WHERE (msgid='" + m['messageid'] + "')";
        var result = connection.executeQuery(query);
      }
      catch (ex)
      {
      }



      ////////// (rough) checking whether mail was inserted properly: //////////
      var msgid = m['messageid'];
      var erroroccured = false;

	  try { var result = connection.executeQuery("select id from header where (msgid ='" + msgid + "')"); }
      catch(ex) { }
	  if(!evmIsEntryAvailableInDB(result)) erroroccured = true;
	  try { var result = connection.executeQuery("select id from body where (msgid ='" + msgid + "')"); }
      catch(ex) { }
	  if(!evmIsEntryAvailableInDB(result)) erroroccured = true;
	  try { var result = connection.executeQuery("select categoryid from header_category where (msgid ='" + msgid + "')"); }
      catch(ex) { }
	  if(!evmIsEntryAvailableInDB(result)) erroroccured = true;
	  try { var result = connection.executeQuery("select mailaddressid from header_mailaddress where (msgid ='" + msgid + "')"); }
      catch(ex) { }
	  if(!evmIsEntryAvailableInDB(result)) erroroccured = true;

	  if(erroroccured == true)
	  {
	  	//alert("The mail with the subject " + m['subject'] + " by " + m['from'] + " to " + m['to'] + " could not be saved properly in all parts, please try again!");
		waitdialog.document.write ("<br>&nbsp;&nbsp;&nbsp;&nbsp; Error occurred");
	  }
	  else
	  {
		// Mail koennte an dieser Stelle geloescht werden sofern gewollt
	  }
      m = {};
    }
    catch(ex)
    {
      alert("EVM Error in function evmDoQuery: " + ex);
    }
  }

// **************************************************************************
// ********************** END of EVM-DB Functions ***************************
// **************************************************************************
